SystemOrganization addCategory: #TusLibros!

Bag subclass: #Cart
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:07'!
productIsNotInTheCatalog
	^'Product not in the catalog'! !

!Cart class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:31'!
withCatalog: aCatalog 
	^self new catalog: aCatalog! !

!Cart methodsFor: 'services' stamp: 'msalegas 5/29/2013 20:37'!
add: aProduct
	(catalog includesKey: aProduct) ifTrue: [super add: aProduct] ifFalse: [ self error: self class productIsNotInTheCatalog]
	! !

!Cart methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 19:33'!
catalog: aCatalog 
	catalog := aCatalog! !

TestCase subclass: #TestCart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 5/30/2013 20:31'!
test01CartShouldBeEmptyWhenCreated
	|catalog cart|
	catalog := Set new.
	cart := Cart withCatalog: catalog.
	self assert: cart isEmpty! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 5/30/2013 20:31'!
test02CartShouldListBooksWhenABookIsAdded
	|catalog cart book|
	book := 'Lord of the Rings'.
	catalog := Dictionary new.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	self assert: (cart includes: book); assert: cart size = 1! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 5/30/2013 20:31'!
test03ABookShouldNotBeAddedToCartIfItIsNotInTheCatalog
	|cart catalog book|
	catalog := Dictionary new.
	cart := Cart withCatalog: catalog.
	book := 'Lord of the Rings'.
	self should: [cart add: book] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cart productIsNotInTheCatalog ]. 
	self assert: cart isEmpty
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 5/30/2013 20:31'!
test04AddABookMultipleTimesMakesTheCartSizeToIncrease
	|cart catalog book|
	book := 'Lord of the Rings'.
	catalog := Dictionary new.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.	
	cart add: book; add: book.
	self assert: (cart size = 2)
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 5/30/2013 20:31'!
test05CartShouldListAllBooksWhenMultipleBooksAreAdded
	|lotr hobbit catalog cart|
	lotr := 'Lord of the Rings'.
	hobbit := 'The Hobbit'.
	catalog := Dictionary new.
	catalog at: lotr put: 20; at: hobbit put: 10.
	cart := Cart withCatalog: catalog.
	cart add: lotr; add: hobbit.
	self assert: (cart includes: lotr); 
		assert: (cart includes: hobbit);
		assert: cart size = 2.! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/3/2013 20:19'!
test06CheckOutWithEmptyCartRaisesErrorAndNoSaleIsDone
	|cart merchantProcessor cashier card sale|
	cart := Cart withCatalog: Set new.
	merchantProcessor := DummyMerchantProcessor new.
	cashier := Cashier date: Date today withPriceList: Set new merchantProcessor: merchantProcessor.
	card := CreditCard new.
	self should: [cashier checkout: cart creditCard: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier invalidCheckOut ].
	sale := cashier lastSale.
	self assert: sale isNil.
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/3/2013 20:36'!
test07CheckOutWithExpiredCreditCardMakeNoSaleAndRaisesErrorAndDoesNotUseMerchantProcessorServices
	|catalog book cart merchantProcessor cashier card |
	catalog := Dictionary new.
	book := 'lotr'.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	merchantProcessor := DummyMerchantProcessor new.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month - 1.
	self should: [ cashier checkout: cart creditCard: card ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier creditCardExpired ]. 
	self assert: cashier lastSale isNil.
	self assert:  (merchantProcessor transactionsFor: cashier) isEmpty
	
	
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/3/2013 21:02'!
test08CheckOutWithCreditCardWithNoCreditMakeNoSaleAndMPTransactionFails
	|catalog book cart merchantProcessor cashier card sale|
	catalog := Dictionary new.
	book := 'lotr'.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	merchantProcessor := DummyMerchantProcessor new.
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	self should: [cashier checkout: cart creditCard: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = MerchantProcessor notEnoughCredit ]. 
	sale := cashier lastSale.
	self assert: sale isNil.
	self deny: (merchantProcessor transactionsFor: card) anyOne result
		
	
	! !

Object subclass: #Cashier
	instanceVariableNames: 'sales priceList date merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error' stamp: 'achavez 5/30/2013 20:07'!
creditCardExpired
	^'Credit card has expired'! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:16'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	 ^self new date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor ! !

!Cashier class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:23'!
invalidCheckOut
	^'Invalid check out'! !

!Cashier methodsFor: 'services' stamp: 'achavez 6/3/2013 20:18'!
checkout: aCart creditCard: aCreditCard 
	| price sale result |
	aCart isEmpty ifTrue: [ ^self error: self class invalidCheckOut ].
	price := 0.
	aCart do: [:product | 
			price := price + priceList at: product.
		].
	aCreditCard expirationDate < date ifTrue: [ ^self error: self class creditCardExpired ].
	result := merchantProcessor debit: price from: aCreditCard.
	result ifTrue: [ 
		sale := Sale cashier: self card: aCreditCard products: aCart contents priceList: priceList.
		sales add: sale.]
		! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:17'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	date := aDate.
	priceList := aPriceList.
	merchantProcessor := aMerchantProcessor.
	sales := OrderedCollection new! !

!Cashier methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:56'!
lastSale
	^sales notEmpty ifTrue: [sales last]! !

Object subclass: #CreditCard
	instanceVariableNames: 'number owner expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:10'!
number: aNumber owner: aString expirationDate: aDate 
	^self new number: aNumber owner: aString expirationDate: aDate ! !

!CreditCard methodsFor: 'services' stamp: 'achavez 5/30/2013 20:18'!
debit: anAmount
	^true! !

!CreditCard methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:17'!
expirationDate
	^expirationDate! !

!CreditCard methodsFor: 'creation' stamp: 'achavez 5/30/2013 20:11'!
number: aNumber owner: aString expirationDate: aDate 
	number := aNumber.
	owner := aString.
	expirationDate := aDate.! !

Object subclass: #MerchantProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

MerchantProcessor subclass: #DummyMerchantProcessor
	instanceVariableNames: 'transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!DummyMerchantProcessor methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 21:01'!
debit: anAmount from: aCard 
	|existing transaction|
	existing := self transactionsFor: aCard.
	transaction := Transaction amount: anAmount card: aCard result: false.
	existing add: transaction.
	transactions at: aCard put: existing.
	^self error: self class notEnoughCredit! !

!DummyMerchantProcessor methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:29'!
initialize
	transactions := Dictionary new.! !

!DummyMerchantProcessor methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:51'!
transactionsFor: aCard 
	^transactions at: aCard ifAbsent: OrderedCollection new! !

!MerchantProcessor class methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:55'!
notEnoughCredit
	^'The card specified has not enough credit'! !

Object subclass: #Sale
	instanceVariableNames: 'cashier card products priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:09'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList
	^self new cashier: aCashier card: aCard products: aCollection priceList: aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:49'!
card
	^ card! !

!Sale methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:48'!
cashier
	^ cashier! !

!Sale methodsFor: 'creation' stamp: 'achavez 6/3/2013 20:10'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList 
	cashier := aCashier.
	card := aCard.
	products := aCollection.
	priceList := aPriceList! !

!Sale methodsFor: 'creation' stamp: 'achavez 5/30/2013 20:58'!
cashier: aCashier date: aDate card: aCard products: aCollection priceList: aPriceList 
	cashier := aCashier.
	date := aDate.
	card := aCard.
	products := aCollection.
	priceList := aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:47'!
date
	^ date! !

!Sale methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:49'!
priceList
	^ priceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:49'!
products
	^ products! !

Object subclass: #Transaction
	instanceVariableNames: 'amount card result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Transaction class methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:53'!
amount: anAmount card: aCard result: aResult 
	^self new amount: anAmount card: aCard result: aResult! !

!Transaction methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 20:53'!
amount: aSmallInteger card: aCreditCard result: aFalse 
	self shouldBeImplemented.! !

!Transaction methodsFor: 'as yet unclassified' stamp: 'achavez 6/3/2013 21:02'!
result
	^ result! !
